/*=================================================================*\
  Example: HAN's maximum rank correlation estimator
  In this DO file you can find two implementations of the MRC.
  One is a faithfull conversion of the original formular into
  stata code called "hanslow". Using this sub-function will
  result in a quadratic increase of the computational steps
  in relation to N. This problem manifolds through the usage
  of the GA. The approach of the sub-function "hanfast" results
  in a linear increase of the computational steps. It only works
  for a binary choice model and uses clever sorting to circumvent
  the nested loops of the faithful "hanslow".
  There is a third version called "hanveryfast". It is an
  implementation of "hanfast" in mata, a matrix programming
  language and integral part of stata.  
  You can compare the performance of all versions by switching the
  comments in the fitness function "han" (default: "hanveryfast").
\*=================================================================*/
clear all
set mem 10m
set more off

set obs 1000
gen double x = invnorm(uniform())
gen double c = invnorm(uniform())^2
gen double u = uniform()
gen double err = exp(invnorm(uniform()))
sum err, d
replace err = err-r(p50)
gen double y = (x - u - c + 1 + err) > 0
	
cap program drop hanfast
program define hanfast, rclass
	local y = "`1'"
	local s = "`2'"
	local rank = 0
	sort `y' `s'
	qui count if `y' == 0
	local y0 = r(N)
	local i = 1
	local j = `y0'+1
	while `i' <= `y0' & `j' <= _N {
		if `s'[`i'] < `s'[`j'] {
			local rank = `rank'+_N-`j'+1
			local i = `i'+1
		}
		else {
			local rank = `rank' + `i'-1
			local j = `j'+1
		}
	}
	if `i' > `y0' {
		local rank = `rank'+(_N-`j'+1)*`y0'
	}
	return scalar rank = `rank'/(_N*(_N-1))
end

mata:
void hanveryfast(string scalar yname, string scalar sname)
{	real matrix han
	real scalar i, j, y0, y01, rank
	han = st_data(.,(yname,sname))
	_sort(han,(1,2))
	y0 = colsum(han[.,1]:==0)
	y01 = rows(han)
	rank = 0
	i = 1
	j = y0 + 1
	while (i <= y0 & j <= y01)
	{	if (han[i,2] < han[j,2])
		{	rank = rank+y01-j+1
			i++
		}
		else
		{	rank = rank+i-1
			j++
		}
	}
	if (i > y0)
	{	rank = rank+(y01-j+1)*y0
	}
    st_numscalar("r(rank)", rank/(y01*(y01-1)))
}
end

cap program drop hanslow
program define hanslow, rclass
	local y = "`1'"
	local s = "`2'"
	local rank = 0
	local i = 1
	while `i' <= _N {
		local j = 1
		while `j' <= _N {
			if `y'[`i'] < `y'[`j'] & `s'[`i'] < `s'[`j'] | `y'[`i'] > `y'[`j'] & `s'[`i'] > `s'[`j'] {
				local rank = `rank'+1
			}
			local j = `j'+1
		}
		local i = `i'+1
	}
	return scalar rank = `rank'/(_N*(_N-1))
end

cap program drop han
program define han
	tempvar xb 
	scalar len = sqrt(dna[1,1]^2+dna[1,2]^2+dna[1,3]^2)
	if len > 0 {
		matrix dna = dna / len  // normalized vector
	}
	qui gen double `xb' = x*dna[1,1] + u*dna[1,2] + c*dna[1,3]
//	hanfast y `xb'
	mata: hanveryfast("y","`xb'")
//  hanslow y `xb'
	qui matrix dna[1,$DNAFIT] = r(rank)
end

dna clear
dna fun han
dna len 3
dna pop 30
dna off 60
dna rad 2 25
dna mut 1 20
dna gen 500
dna sel 1
dna sta 20
for num 1/3 : dna def X -2 2
dna info
dna go
